% MAINSYINTHETIC applies the method described in Benfenati et al, Efficient
% segmentation and positioning of 3D fluorescent spherical beads in
% confocal microscopy. 
%
clearvars
close all
clc

%% Setting
N       = [512,512,22];              % Resolution in voxels
ranges  = [76.800 76.800 7];         % Dimensions in nm
r       = 1;                         % Beads Radius
n       = 100;                       % number of particles
dx      = ranges(1)/N(1);            % Discretization in x axis
dy      = ranges(2)/N(2);            % Discretization in y axis
dz      = ranges(3)/N(3);            % Discretization in z axis
Zx      = ceil(2*r/dz);              % Nr of frames spanned by a particle
sigma_N = 0.2;                       % Gaussian noise
BMsp    = [1 1 1];                   % Brownian motion in the 3 directions
sp      = zeros(n,3);                % Deterministic Speed
TIME    = 1;                         % Number of time frames

flag = 1;
%% Simulation
% TO DO: improve the random positioning
while flag
    iCenter = [3+72.800*rand(n,2), 1+5*rand(n,1)];
    [a,b] = knnsearch(iCenter,iCenter,'k',n-1);
    if sum(sum((b(:,2:end)<2*r)))>0
        flag = 1;
    else
        flag = 0;
    end
end

[C,B,cinfo,CLEAR] = particlesSIM(n,ranges,N,r,...
    'TM',TIME,...
    'SPBM',BMsp,...
    'SP',sp,...
    'dTM',1,...
    'CENTER',iCenter,...
    'sigma',sigma_N,...
    'poisson',1);


%% Options
TV_funct    = @gradient;
window      = 0.5;
H           = fspecial('Gaussian',5,1);
rhoMerging  = 0.1*r;
rhoShrink   = 0.1*r;


% Cell Array with the final estimation of the center
R = cell(TIME,1);

%% ESTIMATION
for t = 1:TIME
    fprintf('TIME: %2d of %2d - frame: ',t,TIME);
    % Allocating names: C will contain the candidates for the centers,
    % while cRaw will be used to compute the first raw estimation
    C    = [];
    cRaw = [];
    
    for z = 1:N(3)
        fprintf('%2d of %2d',z,N(3));
        
        imcurrent = B(:,:,z);
        
        if sigma_N>0
            imcurrent = imfilter(imcurrent,H);
        end
        
        % Just a check to be sure that there is something of interest in
        % the image
        if max(imcurrent(:))>40
            
            thres_lb = graythresh(imcurrent/max(imcurrent(:)));
            % To avoid a too low threshold
            thres_lb = thres_lb*(thres_lb>=0.3) + 0.3*(thres_lb<0.3);
            
            [Ccurrent, Rcurrent ,w] = labelLSimage(imcurrent,[dx dy],r,...
                'TLB',thres_lb,...
                'TV',TV_funct,...
                'WIN',window);
            
            % The candidates
            C = [C; Ccurrent, z*dz+sqrt(r^2-Rcurrent.^2), w
                Ccurrent, z*dz-sqrt(r^2-Rcurrent.^2), w];
            % The coordinates of the centers in each frame
            cRaw = [cRaw ; Ccurrent, z*dz*ones(size(Rcurrent)), Rcurrent];
        end
        
        if z<N(3)
            fprintf('\b\b\b\b\b\b\b\b');
        end
    end
    
    % Computing a first raw estimation of the centers
    [CENTERS,RADII] = mergingSPHERE(cRaw,rhoMerging,r);
    % Storing in R{t} the final correct estimation
    R{t}            = shrinkCenters_VER2(C,CENTERS,rhoShrink);
    fprintf('\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b');
end

rtot = R{1};

%%  COMPUTING ERRORS
D = diag(1./[dx dy dz].^2);

errxyR   = zeros(n,1);
errzR    = zeros(n,1);
errR     = zeros(n,1);
iCenter  = iCenter(:,[2,1 3]);
for k = 1:n
    [a,b] = knnsearch(rtot,iCenter(k,:),'k',1);
    if ~isempty(a) && b<0.5
        errxyR(k)     = sqrt((rtot(a,1:2)-iCenter(k,1:2))*D(1:2,1:2)*(rtot(a,1:2)-iCenter(k,1:2))');
        errzR(k)      = abs((rtot(a,3)-iCenter(k,3)))/dz;
        errR(k)       = sqrt(errxyR(k)^2+errzR(k)^2);

        rtot(a,:)   = [];
    else
        errxyR(k) = -inf;
        errzR(k)  = -inf;
        errR(k)   = -inf;
    end
end


fprintf('Recognized by knn: %d\n',size(R{1},1))
fprintf('T : %0.4f\nV : %0.4f\nP : %0.4f\n',...
    mean(errR(~isinf(errR))),mean(errzR(~isinf(errzR))),mean(errxyR(~isinf(errxyR))))

%==========================================================================
%
% Version : 1.2 (06-06-2019)
% Author  : A. Benfenati (alessandro.benfenati@unimi.it)
%
%==========================================================================
%
% COPYRIGHT NOTIFICATION
%
% Permission to copy and modify this software and its documentation for
% internal research use is granted, provided that this notice is retained
% thereon and on all copies or modifications. The authors and their
% respective Universities makes no representations as to the suitability
% and operability of this software for any purpose. It is provided "as is"
% without express or implied warranty. Use of this software for commercial
% purposes is expressly prohibited without contacting the authors.
%
% This program is free software; you can redistribute it and/or modify it
% under the terms of the GNU General Public License as published by the
% Free Software Foundation; either version 3 of the License, or (at your
% option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
% Public License for more details.
%
% You should have received a copy of the GNU General Public License along
% with this program; if not, either visite http://www.gnu.org/licenses/ or
% write to Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA
% 02139, USA.
%


