function [CV, RV, w] = labelLSimage(I,d,r,varargin)
% LABELLSIMAGE estimates the centers of particles in the given
% image I.
%
%   [CV, RV, w] = labelLSimage(I,d,r,varargin)
%
% INPUT
%
% I         : (double array) image
% d         : (double array) 2D array containing the discretization steps
%                            dx and dy
% r         : (double)       radius of the particles
%
% OUTPUT
%
% CV        : (double array) list of estimated centers
% RV        : (double array) list of estimated radii
% w         : (double array) number of pixels for each circle
%
%==========================================================================
%
% Version : 1.2 (06-06-2019)
% Author  : A. Benfenati (alessandro.benfenati@unimi.it)
%
%==========================================================================
%
% COPYRIGHT NOTIFICATION
%
% Permission to copy and modify this software and its documentation for
% internal research use is granted, provided that this notice is retained
% thereon and on all copies or modifications. The authors and their
% respective Universities makes no representations as to the suitability
% and operability of this software for any purpose. It is provided "as is"
% without express or implied warranty. Use of this software for commercial
% purposes is expressly prohibited without contacting the authors.
%
% This program is free software; you can redistribute it and/or modify it
% under the terms of the GNU General Public License as published by the
% Free Software Foundation; either version 3 of the License, or (at your
% option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
% Public License for more details.
%
% You should have received a copy of the GNU General Public License along
% with this program; if not, either visite http://www.gnu.org/licenses/ or
% write to Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA
% 02139, USA.
%


T_lb = 0.3;
win  = 0.2;
tv_f = @gradient;

if (nargin-length(varargin)) ~= 3
    error('Wrong number of required parameters');
end

if (rem(length(varargin),2)==1)
    error('Optional parameters should always go by pairs');
    fprintf('!\n');
else
    for i=1:2:(length(varargin)-1)
        switch upper(varargin{i})
            case 'TLB'
                T_lb = varargin{i+1};
            case 'WIN'
                win = varargin{i+1};
            case 'TV'
                tv_f = varargin{i+1};
        end
    end
end

dx = d(1);
dy = d(2);

w  = [];
CV = [];
RV = [];
WD = [];
In = [];


%% Labeling
TI                      = zeros(size(I));
TI(I>T_lb*max(I(:)))    = 1;
[L,NUM]                 = bwlabel(TI,8);


if any(NUM)
    [Gx,Gy]         = feval(tv_f,I,dx,dy);
    Q               = sqrt(Gx.^2+Gy.^2);
    for K = 1:NUM

        [i,j]   = find(L==K);
        i       = i*dx;
        j       = j*dy;
        c       = round([ mean(i)/dx, mean(j)/dy]);
        
        % Create the window of size
        % (1+win) * dist(mass center, farest away labeled pixel)
        [~,b]   = knnsearch([i,j],[mean(i), mean(j)],'k',length(i));
        WD = [WD; numel(i)];
        rr      = round(((1+win)*max(b)/dx));
        
        xint    = c(1)-rr:c(1)+rr;
        yint    = c(2)-rr:c(2)+rr;
        
        while any(xint<=0) || any(xint>size(I,1))
            rr   = rr-1;
            xint = c(1)-rr:c(1)+rr;
        end

        while  any(yint<=0) ||  any(yint>size(I,2))
            rr = rr-1;
            yint    = c(2)-rr:c(2)+rr;
        end
        
        %fprintf('Correct woi.\n')
        if any([xint,yint])
            Z               = zeros(size(I));
            Z(xint,yint)    = Q(xint,yint);
            thres           = graythresh(Q/max(Q(:)))*max(Q(:));
            thresQ          = zeros(size(Q));
            thresQ(Z>thres) = 1;
            
            %% Check multiple connected components
            [Lbis, Nbis] = bwlabel(thresQ,8);
            if Nbis <2
                [ii,jj]         = find(thresQ>0);
            else
                ncomp   = 1;
                area    = sum(Lbis(:)==ncomp);
                for ncp = 2:Nbis
                    if max(area,sum(Lbis(:)==ncp))~=area
                        area = sum(Lbis(:)==ncp);
                        ncomp = ncp;
                    end
                    
                end
                [ii,jj]         = find(Lbis==ncomp);
            end
            
            W               = [];
            for f = 1:length(ii)
                W(f)        = Q(ii(f),jj(f));
            end
            cand            = [ii*dx,jj*dy];
            
            if size(cand,1)>5
                [cest,rest] = LSsphere(cand,'constrain',(0.999*r)^2,'W',W');
                CV = [CV; cest'];
                RV = [RV;rest];
                w  = [w; numel(i)];
            end
                      
            
        end
    end
    
    
end
