function [CENTERS,RADII] = mergingSPHERE(C,rho,r)
% MERGINGSPHERE computes a raw estimation of spheres via a LS fit.
%
%   [CENTERS,RADII] = mergingSPHERE(C,rho,r)
%
%  It uses the profiles and the centers obtained by the analysis of frames
%  from a 3D scanned volume. It creates cluster of profiles by searching in
%  C the points that are lying in a range equal to rho ONLY in 2D plane.
%  Once the centers are correctly labeled, the profiles are computed and
%  then used for the estimation of the sphere.
%
%  INPUT
% 
%  C       : (double array) The array containing the n estimation of the
%                           center in the frames. The format has to be the
%                           following: the first 3 columns must contain the
%                           3D coordinates of the centers, the4th must
%                           contain the estimated radius of the profile of
%                           theparticle.
%
%  rho     : (double) distance within which the center are recognized to
%                     belong to the same sphere
%
%  r       : (double) initially estimation of the radius of the particle.
%                     It is used for the 3D raw estimation
%
% OUTPUT
%
%  CENTERS : (double array)  3D coordinates of the m recognized particles.
%
%  RADII   : (double array)  estimated radius of each particle.
%
% See also LSsphere, shrinkCenters_VER2
%
%==========================================================================
%
% Version : 1.2 (06-06-2019)
% Author  : A. Benfenati (alessandro.benfenati@unimi.it)
%
%==========================================================================
%
% COPYRIGHT NOTIFICATION
%
% Permission to copy and modify this software and its documentation for
% internal research use is granted, provided that this notice is retained
% thereon and on all copies or modifications. The authors and their
% respective Universities makes no representations as to the suitability
% and operability of this software for any purpose. It is provided "as is"
% without express or implied warranty. Use of this software for commercial
% purposes is expressly prohibited without contacting the authors.
%
% This program is free software; you can redistribute it and/or modify it
% under the terms of the GNU General Public License as published by the
% Free Software Foundation; either version 3 of the License, or (at your
% option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
% Public License for more details.
%
% You should have received a copy of the GNU General Public License along
% with this program; if not, either visite http://www.gnu.org/licenses/ or
% write to Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA
% 02139, USA.
%


if nargin<3
    error('Missing argument: type ''help mergingSPHERE'' for infos.')
end

% Allocating names
oldCENTERS = []; % Temporary centers
CENTERS    = []; % Final centers
RADII      = []; % Estimated radii

% Variables used to regroups the centers
indexes     = []; 
label       = 1; 

% Profile points
x = cos(linspace(0,2*pi))';
y = sin(linspace(0,2*pi))';

%% Grouping
while ~isempty(C)
    % Only the xy coordinates are used 
    cand = C(1,1:2);
    indexes = [indexes; label, C(1,:)];
    to_delete = 1;
    for i = 2:size(C(2:end,:),1)
        if norm(cand-C(i,1:2))<rho
            indexes = [indexes; label, C(i,:)];
            to_delete = [to_delete, i];
        end
    end
    C(to_delete,:)  = [];
    label = label +1;
end

%% LS Estimation
for i = 1:label-1
    % Points used for the LS estimation
    points  = [];
    W       = [];
    idx     = find(indexes(:,1)==i);
    if length(idx)>2
        for K = 1:length(idx)
            points = [points;...
                indexes(idx(K),2)+indexes(idx(K),5)*x,...
                indexes(idx(K),3)+indexes(idx(K),5)*y,...
                indexes(idx(K),4)*ones(size(x))];
            W = [W;pi*indexes(idx(K),5)^2.*ones(size(x))];
        end
        
        [center,radius] = LSsphere(points,'constrain',r^2,'W',W);
        % Store the estimated center and radius
        oldCENTERS      = [oldCENTERS; center'];
        RADII           = [RADII; radius];
    end
end

%% Check if some estimations lead to collapsing spheres
[nearIDX,nearDST]=rangesearch(oldCENTERS,oldCENTERS,r);

for k = 1:length(nearDST)
    if sum(nearDST{k}>0)>0
        % If some estimated centers seems to lead to collapsing spheres,
        % the mean of the coordinates are taken
        CENTERS = [CENTERS; mean(oldCENTERS(nearIDX{k},:))];
    else 
        CENTERS = [CENTERS; oldCENTERS(k,:)];
    end
end
